/* schur_invssts_c.c: Generic Mex-C source code for the State-Space Time-Series
 * Gaussian Process Prior Models.
 *
 * Syntax used:
 *				[Q,logdet,tr,invB] = schur_invssts_c(G,ord,b);
 *
 * Inputs:
 *			G   : generating matrix, size (2n)x(2rk)
 *			ord : order of matrix transformation
 *			b   : vector for inv(T)*b
 *
 * Outputs:
 *			Q       = returns N*inv(T)*N', where N is a permuting matrix
 *			logdet  = log(det(T))
 *			tr      = trace(inv(T))
 *			invB    = returns column vector of inv(T)*b
 *
 * Generates the Generalised Schur's complement of
 * the Hermitian matrix, R, such that
 *
 * 		R=[-T  I]
 * 		  [ I  0]
 *
 * where T is the positive-definite Toeplitz-like Block matrix. Note
 * that no hyperbolic rotation is implemented.
 *
 * For use with Matlab version 7.0 (tested) and above. Other versions
 * may be possible, but not tested.
 *
 * Instruction: Run <mex xxx.c> at Matlab command prompt to compile
 * the .c code into .dll (Windows) or .meglx (Linux/UNIX).
 *
 *
 * (C) Gaussian Process SSTS Toolbox 1.0
 * (C) Copyright 2004-2007, Keith Neo <keith.neo@nuim.ie>
 * 							http://www.hamilton.ie */

#include <math.h>
#include "mex.h"
#define abs(x) (((x)>0)?(x):-(x))


void trace_invfast3(unsigned long n,unsigned long rk,unsigned long d0,double *ord,double *G,double *b,double *invQ,double *logdet,double *trq2,double *invb)
{
	unsigned long k,j,h,n2,np,nk,nz,nx,nr,dx,rk2;
	unsigned long *ind;
	double tmp,tmp1,d;
	double *Gen,*g,*l;

	n2=n/2;
	Gen=(double *)malloc(n*rk*sizeof(double));
	for (k=0;k<rk;k++)
	{
		nz=k*n;
		for (j=0;j<n;j++)
		{
			np=nz+j;
			if (j<=n2) Gen[np]=G[np];
			else Gen[np]=0.0;
		}
	}

	np=(unsigned long)ord[0]+1;
	ind=(unsigned long *)malloc(d0*sizeof(unsigned long));
	ind[0]=0;
	for (k=1;k<d0;k++) ind[k]=ind[k-1]+np;

	dx=1;
	nx=n2+2;
	rk2=rk/2;
	logdet[0]=0.0;
	l=(double *)malloc(n*sizeof(double));
	g=(double *)malloc(rk*sizeof(double));
	for (k=0;k<n2;k++)
	{
		nk=n-k;
		for (j=0;j<rk;j++) g[j]=Gen[j*nk];

		for (j=0;j<nk;j++)
		{
			tmp=0.0;
			for (h=0;h<rk;h++)
			{
				d=Gen[h*nk+j]*g[h];
				if (h<rk2) tmp=tmp+d;
				else tmp=tmp-d;
			}
			l[j]=tmp;
		}
		logdet[0]=logdet[0]+log(abs(l[0]));
		d=-1.0/l[0];

		if ((dx<d0) && (k==ind[dx])) dx=dx+1;

		nr=nk-n2;
		for (j=0;j<dx;j++)
		{
			np=j*d0;
			nz=ind[j];
			tmp=l[nr+nz]*d;
			for (h=0;h<=j;h++)
			{
				nz=ind[h];
				invQ[np+h]=invQ[np+h]+tmp*l[nr+nz];
			}
		}

		tmp=b[k];
		tmp1=1.0;
		for (j=0;j<k;j++)
		{
			np=nr+j;
			tmp=tmp+l[np]*b[j];
			tmp1=tmp1+l[np]*l[np];
		}
		trq2[0]=trq2[0]+tmp1*d;
		tmp1=tmp*d;
		invb[k]=tmp1;
		for (j=0;j<k;j++) invb[j]=invb[j]+l[nr+j]*tmp1;

		for (j=(nk-1);j>0;j--)
		{
			if (j==nr) l[j]=-l[j];
			else l[j]=l[j-1]-l[j];
		}

		for (j=0;j<rk;j++)
		{
			tmp=g[j]*d;
			nz=j*nk;
			nr=nz-j-1;
			for (h=1;h<nk;h++)
			{
				if (h<nx) Gen[nr+h]=Gen[nz+h]-l[h]*tmp;
				else Gen[nr+h]=0.0;
			}
		}
	}
	free(g);
	free(l);
	free(Gen);
	free(ind);
	for (k=1;k<d0;k++)
	{
		np=k*d0;
		for (h=0;h<k;h++) invQ[h*d0+k]=invQ[np+h];
	}
	return;
}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
	double *INVQ;
	double *LOGDET;
	double *TR;
	double *INVB;
	double *G;
	double *ORD;
	double *B;
	unsigned long N;
	unsigned long R;
	unsigned long D0;

  if(nrhs!=3) {
    mexErrMsgTxt("Three inputs required.");
  } else if(nlhs>4) {
    mexErrMsgTxt("Too many  output arguments");
  }

  N = mxGetM(prhs[0]);
  R = mxGetN(prhs[0]);

  G   	 = mxGetPr(prhs[0]);
  ORD 	 = mxGetPr(prhs[1]);
  B   	 = mxGetPr(prhs[2]);

  D0=(unsigned long)ceil(N/(2*(ORD[0]+1)));

  plhs[0] = mxCreateDoubleMatrix(D0,D0, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
  plhs[2] = mxCreateDoubleMatrix(1,1, mxREAL);
  plhs[3] = mxCreateDoubleMatrix(N/2,1, mxREAL);

  INVQ 	 = mxGetPr(plhs[0]);
  LOGDET = mxGetPr(plhs[1]);
  TR 	 = mxGetPr(plhs[2]);
  INVB 	 = mxGetPr(plhs[3]);

  trace_invfast3(N,R,D0,ORD,G,B,INVQ,LOGDET,TR,INVB);
  return;
}
