/* schur_invssts_cz.c: Generic Mex-C source code for the State-Space Time-Series
 * Gaussian Process Prior Models.
 *
 * Syntax used:
 *				[invQ,NQf] = schur_invssts_cz(G,ord,n);
 *
 * Inputs:
 *			G   : generating matrix, size (N0+Nt+1)x(rk+rk_n+1)x2
 *			ord : order of matrix transformation
 *			n   : original data size
 *
 * Outputs:
 *			invQ	= returns diagonal of A'*inv(T)*A
 *			NQf		= returns the product of N*inv(T)*A
 *
 * Generates the fast algorithm for finding
 * the standard deviation of the State-Space Time-Series Gaussian Process
 * Prior Model using Generalised Schur algorithm. Two outputs are obtained
 * from this aspect.
 *
 * 		     [-T   A   I]
 * 		 R = [ A'  0   0]
 * 		     [ I   0   0]
 *
 * where T is positive-definite strongly Toeplitz-like Block and dQ is the
 * cross-covariance matrix between the explanatory variables and the
 * predicted explanatory variables.
 *
 * For use with Matlab version 7.0 (tested) and above. Other versions
 * may be possible, but not tested.
 *
 * Instruction: Run <mex xxx.c> at Matlab command prompt to compile
 * the .c code into .dll (Windows) or .meglx (Linux/UNIX).
 *
 *
 * (C) Gaussian Process SSTS Toolbox 1.0
 * (C) Copyright 2004-2007, Keith Neo <keith.neo@nuim.ie>
 * 							http://www.hamilton.ie */

#include <math.h>
#include "mex.h"

void ssts_std(unsigned long n,unsigned long rk,unsigned long d0,double *G,double *ordx,unsigned long n3,double *invQ,double *NQf)
{
	unsigned long k,j,h,np,nk,nz,nr,nw,ns,nm,rk2,ord;
	double tmp,d;
	double *Gen,*g,*l;

	Gen=(double *)malloc(n*rk*sizeof(double));
	for (k=0;k<rk;k++)
	{
		nz=k*n;
		for (j=0;j<n;j++)
		{
			np=nz+j;
			Gen[np]=G[np];
		}
	}

	/*prediction size*/
	nr=n-2*n3;
	rk2=rk/2;
	nw=1;
	ord=(unsigned long)ordx[0];
	l=(double *)malloc(n*sizeof(double));
	g=(double *)malloc(rk*sizeof(double));
	for (k=0;k<n3;k++)
	{
		nk=n-k;
		for (j=0;j<rk;j++) g[j]=Gen[j*nk];

		for (j=0;j<nk;j++)
		{
			tmp=0.0;
			for (h=0;h<rk;h++)
			{
				d=Gen[h*nk+j]*g[h];
				if (h<rk2) tmp=tmp+d;
				else tmp=tmp-d;
			}
			l[j]=tmp;
		}
		d=-1.0/l[0];

		nz=n3-k;
		for (j=0;j<nr;j++)
		{
			np=nz+j;
			invQ[j]=invQ[j]+d*l[np]*l[np];
		}
		nm=0;
		np=nz+nr;
		for (j=0;j<nw;j++)
		{
			tmp=d*l[np+nm+j];
			for (h=0;h<nr;h++)
			{
				ns=h*d0+j;
				NQf[ns]=NQf[ns]+l[nz+h]*tmp;
			}
			nm=nm+ord;
		}
		if (k==nw*(ord+1)-1) nw=nw+1;

		for (j=(nk-1);j>0;j--)
		{
			if ((j==nz) || (j==np)) l[j]=-l[j];
			else l[j]=l[j-1]-l[j];
		}

		for (j=0;j<rk;j++)
		{
			tmp=g[j]*d;
			nz=j*nk;
			np=nz-j-1;
			for (h=1;h<nk;h++) Gen[np+h]=Gen[nz+h]-l[h]*tmp;
		}
	}
	free(g);
	free(l);
	free(Gen);
	return;
}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
	double *INVQ;
	double *NQF;
	double *G;
	double *ORD;
	double *N3;
	unsigned long NG;
	unsigned long R;
	unsigned long D0;
	unsigned long N;

  if(nrhs!=3) {
    mexErrMsgTxt("Three inputs required.");
  } else if(nlhs>2) {
    mexErrMsgTxt("Too many  output arguments");
  }

  NG = mxGetM(prhs[0]);
  R  = mxGetN(prhs[0]);

  G   	 = mxGetPr(prhs[0]);
  ORD 	 = mxGetPr(prhs[1]);
  N3	 = mxGetPr(prhs[2]);

  N=(unsigned long)N3[0];
  D0=(unsigned long)ceil(N/(ORD[0]+1));

  plhs[0] = mxCreateDoubleMatrix(NG-2*N,1, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(D0,NG-2*N, mxREAL);

  INVQ 	 = mxGetPr(plhs[0]);
  NQF 	 = mxGetPr(plhs[1]);

  ssts_std(NG,R,D0,G,ORD,N,INVQ,NQF);
  return;
}
