/* schur_invssts_qq_c.c: Generic Mex-C source code for the State-Space Time-Series
 * Gaussian Process Prior Models.
 *
 * Syntax used:
 *				[QQ] = schur_invssts_qq_c(G,ord);
 *
 * Inputs:
 *			G   : generating matrix, size (3n)x(2rk)
 *			ord : order of matrix
 *
 * Outputs:
 *			QQ	= returns N*{ (v/a)*inv(T*T) + inv(T) }*N'
 *
 * Generates the Generalised Schur's complement of the
 * Hermitian matrix, R, such that
 *
 * 		    [ T+(v/a)I    T  I ]
 * 		R = [     T       0  I ]
 * 		    [     0       I  0 ]
 *
 * where T is a positive-definite strongly Toeplitz-like Block  matrix. Note
 * that no hyperbolic rotation is implemented.
 *
 * For use with Matlab version 7.0 (tested) and above. Other versions
 * may be possible, but not tested.
 *
 * Instruction: Run <mex xxx.c> at Matlab command prompt to compile
 * the .c code into .dll (Windows) or .meglx (Linux/UNIX).
 *
 *
 * (C) Gaussian Process SSTS Toolbox 1.0
 * (C) Copyright 2004-2007, Keith Neo <keith.neo@nuim.ie>
 * 							http://www.hamilton.ie */

#include <math.h>
#include "mex.h"

void trace_schur3qq(unsigned long n,unsigned long rk,unsigned long d0,double *ord,double *G,double *invQ)
{
	unsigned long k,j,h,n3,np,nk,nz,nx,nr,dx,rk2;
	unsigned long *ind;
	double tmp,d;
	double *Gen,*l,*g;

	n3=n/3;
	nx=n-n3;
	Gen=(double *)malloc(n*rk*sizeof(double));
	for (k=0;k<rk;k++)
	{
		nz=k*n;
		for (j=0;j<n;j++)
		{
			np=nz+j;
			if (j<=nx) Gen[np]=G[np];
			else Gen[np]=0.0;
		}
	}

	np=(unsigned long)ord[0]+1;
	ind=(unsigned long *)malloc(d0*sizeof(unsigned long));
	ind[0]=0;
	for (k=1;k<d0;k++) ind[k]=ind[k-1]+np;

	dx=1;
	nx=nx+2;
	rk2=rk/2;
	l=(double *)malloc(n*sizeof(double));
	g=(double *)malloc(rk*sizeof(double));
	for (k=0;k<2*n3;k++)
	{
		nk=n-k;
		for (j=0;j<rk;j++) g[j]=Gen[j*nk];

		for (j=0;j<nk;j++)
		{
			tmp=0.0;
			for (h=0;h<rk;h++)
			{
				d=Gen[h*nk+j]*g[h];
				if (h<rk2) tmp=tmp+d;
				else tmp=tmp-d;
			}
			l[j]=tmp;
		}
		d=1.0/l[0];

		nr=nk-n3;
		if (k>=n3)
		{
			h=k-n3;
			if ((dx<d0) && (h==ind[dx])) dx=dx+1;

			for (j=0;j<dx;j++)
			{
				np=j*d0;
				nz=ind[j];
				tmp=l[nr+nz]*d;
				for (h=0;h<=j;h++)
				{
					nz=ind[h];
					invQ[np+h]=invQ[np+h]-tmp*l[nr+nz];
				}
			}
		}

		/* l[0] does not matter at all */
		np=nr-n3;
		for (j=(nk-1);j>0;j--)
		{
			if ((j==nr) || (j==np)) l[j]=-l[j];
			else l[j]=l[j-1]-l[j];
		}

		for (j=0;j<rk;j++)
		{
			tmp=g[j]*d;
			nz=j*nk;
			nr=nz-j-1;
			for (h=1;h<nk;h++)
			{
				if (h<nx) Gen[nr+h]=Gen[nz+h]+l[h]*tmp;
				else Gen[nr+h]=0.0;
			}
		}
	}
	free(g);
	free(l);
	free(Gen);
	free(ind);
	for (k=1;k<d0;k++)
	{
		np=k*d0;
		for (h=0;h<k;h++) invQ[h*d0+k]=invQ[np+h];
	}
	return;
}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
	double *INVQ;
	double *G;
	double *ORD;
	unsigned long D0;
	unsigned long N;
	unsigned long R;

  if(nrhs!=2) {
    mexErrMsgTxt("Two inputs required.");
  } else if(nlhs>1) {
    mexErrMsgTxt("Too many  output arguments");
  }

  N = mxGetM(prhs[0]);
  R = mxGetN(prhs[0]);

  G   	= mxGetPr(prhs[0]);
  ORD	= mxGetPr(prhs[1]);

  D0=(unsigned long)ceil(N/(3*(ORD[0]+1)));

  plhs[0] = mxCreateDoubleMatrix(D0,D0, mxREAL);

  INVQ 	= mxGetPr(plhs[0]);

  trace_schur3qq(N,R,D0,ORD,G,INVQ);
  return;
}
